//-----------------------------------------------------------------------------
//
// Tetris Extreme - A work in progress
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Header files
//-----------------------------------------------------------------------------

#include "core.h"


struct PIECE{
	char piece[4][4],type;
	int x,y;
};

//----------------------------------------------------------------------------
// Functions
//----------------------------------------------------------------------------

void DrawBoard();
void NextPiece();
void RotatePiece();
void right();
void left();
int Update();

//----------------------------------------------------------------------------
// Global variables
//----------------------------------------------------------------------------

LPDIRECT3DTEXTURE8 blocks;
PIECE currentPiece;
PIECE nextPiece;
int board[10][22];

//----------------------------------------------------------------------------
// InitD3D() : Initializes Direct3D
//----------------------------------------------------------------------------

HRESULT Init( HWND hWnd )
{
	int i,j;
	InitD3D(hWnd);

	// Seting up data
	blocks = LoadTexture("blocks.bmp");

	srand(238);
	//Init Data
	for(i=0;i<10;i++)
		for(j=0;j<22;j++)
			board [i][j] = 0;
	
	NextPiece();
	NextPiece();

	dx8.pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER , 
	D3DCOLOR_XRGB(0,0,0), 1.0f, 0 );

	return S_OK;
}

//----------------------------------------------------------------------------
// UpdateStuff() : Updates changes in the world
//----------------------------------------------------------------------------
void UpdateStuff() {
	static int ud=0;

	ud++;
	if(ud==5){
		Update();
		ud=0;
	}
}

//----------------------------------------------------------------------------
// DrawScene() : Draws the contents of the scene
//----------------------------------------------------------------------------
void DrawScene()
{
	// Drawing code goes here...

	DrawBoard();
	return;
}

//----------------------------------------------------------------------------
// Render() : Prepares for drawing the scene
//----------------------------------------------------------------------------
void Render()
{
	if( dx8.pd3dDevice == NULL)
		return;
	
	// Clear the backbuffer to a black color
	dx8.pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET /*| D3DCLEAR_ZBUFFER */, 
		D3DCOLOR_XRGB(0,0,0), 1.0f, 0 );
	
	// Begin the scene
	dx8.pd3dDevice->BeginScene();
	
	// Rendering of scene objects
	DrawScene();
	
	// End the scene
	dx8.pd3dDevice->EndScene();
	
	// Present the backbuffer contents to the display
	dx8.pd3dDevice->Present( NULL, NULL, NULL, NULL );
}


//----------------------------------------------------------------------------
// MsgProc() : The window's message handler
//----------------------------------------------------------------------------
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg ){
		case WM_KEYUP: 
			switch (wParam) { 
				case VK_ESCAPE:
					PostQuitMessage( 0 );
                    break;
				case VK_UP:
					RotatePiece();
					break;
				case VK_DOWN:
					Update();
					break;
				case VK_LEFT:
					left();
					break;
				case VK_RIGHT:
					right();
					break;
            } 
            return 0;
		case WM_CHAR:
			return 0;
		case WM_DESTROY: // If you use Windowed mode process loss of it
			PostQuitMessage( 0 );
			return 0;
		
		case WM_PAINT:
			// NOTE: In a full-screen app, you may choose to draw the screen at
			// a different point, such as using a timer message or outside of
			// the windows message loop
	        Render();
			ValidateRect( hWnd, NULL );
			return 0;
	}
	return DefWindowProc( hWnd, msg, wParam, lParam );
}


//----------------------------------------------------------------------------
// WinMain() : The application's entry point
//----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
	MSG mssg;                // message from queue
	LONGLONG cur_time;       // current time
	DWORD  time_count=1;    // ms per frame, default if no performance counter
	LONGLONG perf_cnt;       // performance timer frequency
	LONGLONG next_time=40;    // time to render next frame
	BOOL move_flag=TRUE;     // flag noting if we have moved yet

	// Register the window class
	WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_HREDRAW | CS_VREDRAW, MsgProc, 0, 
		sizeof(DWORD), GetModuleHandle(NULL), NULL, LoadCursor(NULL, IDC_ARROW), 
		(HBRUSH) GetStockObject(BLACK_BRUSH), NULL, "Tetris Extreme", NULL };
	RegisterClassEx( &wc );
	
	// Create the application's window
	HWND hWnd = CreateWindow( "Tetris Extreme", "Tetris Extreme", 
		WS_POPUP, 100, 100, 640, 480,
		GetDesktopWindow(), NULL, wc.hInstance, NULL );
	
	// Initialize Direct3D
	if( SUCCEEDED( Init( hWnd ) ) )
	{
		// Show the window
		ShowWindow( hWnd, SW_SHOWDEFAULT );
		UpdateWindow( hWnd ); // Forces a WM_PAINT
		
		if (QueryPerformanceFrequency((LARGE_INTEGER *) &perf_cnt)) {
			// yes, set time_count and timer choice flag
			time_count=perf_cnt/25;  // calculate time per frame based on frequency
			QueryPerformanceCounter((LARGE_INTEGER *) &next_time);
		} else {
			return FALSE;
		}

		// prime the message structure
		PeekMessage( &mssg, NULL, 0, 0, PM_NOREMOVE);
		// run till completed
		while (mssg.message!=WM_QUIT) {
			if (PeekMessage( &mssg, NULL, 0, 0, PM_REMOVE)) {
				TranslateMessage(&mssg);
				DispatchMessage(&mssg);
			} else {
				if (move_flag) { //update world
					UpdateStuff();
					move_flag=FALSE;
				}
				else{
					QueryPerformanceCounter((LARGE_INTEGER *) &cur_time);
					if (cur_time>next_time) { //Is it time to draw
						Render();
						next_time += time_count;
						if (next_time < cur_time)
							next_time = cur_time + time_count;
						move_flag=TRUE;
					}
				}
			}
		}
	}
	
	// Clean up everything and exit the app
	Cleanup();
	UnregisterClass( "Tetris Extreme", wc.hInstance );
	return 0;
}

void DrawBoard(){
	int x,y;
	int tboard [10] [22];
	RECT dest,source;
	source.top = 0;
	source.bottom = 14;
	
	for (x=0;x<10;x++)
		for (y=0;y<22;y++)
			tboard[x][y]=board[x][y];
		
	for (x=0;x<4;x++)
		for (y=0;y<4;y++)
			if ((currentPiece.x+x<10)&&(currentPiece.y+y<22)&&(currentPiece.piece[x][y]!=0))
				tboard[x+currentPiece.x][y+currentPiece.y]=currentPiece.piece[x][y];
				
	for(y=19;y>=0;y--){
		dest.top = y*15;
		dest.bottom = y*15+15;
			for(x=0;x<10;x++)
				if (tboard[x][y+2]!=0){
					dest.left = x*16;
					dest.right = x*16+16;
					source.left = 0+16*(tboard[x][y+2] - 2);
					source.right = 15+16*(tboard[x][y+2] - 2);
					BltSpriteEx(&dest,blocks,&source,0);
				}
	}
}

void NextPiece()
{
	int x,y,i,j,temp;

	currentPiece=nextPiece;

	for (x=0;x<4;x++)
		for (y=0;y<4;y++)
			nextPiece.piece[x][y]=0;
    temp = rand()%7+1;
//	temp = 5;
	switch (temp){
		case 1:     //line
			for (x=0;x<4;x++)
				nextPiece.piece[x][1]=2;
			nextPiece.type = 1;
			break;
		case 2:       //L
			for (x=1;x<4;x++)
				nextPiece.piece[x][2]=3;
			nextPiece.piece[1][1]=3;
			nextPiece.type = 2;
			break;
		case 3:       //BL
			for (x=1;x<4;x++)
				nextPiece.piece[x][2]=4;
			nextPiece.piece[2][1]=4;
			nextPiece.type = 2;
			break;
		case 4:      //T
			for (x=1;x<4;x++)
				nextPiece.piece[x][2]=5;
			nextPiece.piece[3][1]=5;
			nextPiece.type = 2;
			break;
		case 5:  // Square
			for (i=1;i<3;i++)
				for (j=1;j<3;j++)
					nextPiece.piece[i][j]=6;
				nextPiece.type = 3;
		break;
		case 6: // z
			for (i=0;i<2;i++){
				nextPiece.piece[i][1]=7;
				nextPiece.piece[i+1][2]=7;
			}
			nextPiece.type = 1;
		break;
		case 7:// s
			for (i=0;i<2;i++){
				nextPiece.piece[i+1][1]=8;
				nextPiece.piece[i][2]=8;
			}
			nextPiece.type = 1;
		break;
	}
	currentPiece.x=3;
	currentPiece.y=0;
}

int Update()
{
	int x,y,i,change_all=0,game_over=0,loop=1,line=0;
	int tboard [10] [22];

	for (x=0;x<10;x++)
		for (y=0;y<22;y++)
			tboard[x][y]=board[x][y];

	for (x=0;x<4;x++)
		for (y=0;y<4;y++)
			if((currentPiece.piece[x][y]!=0)&&(currentPiece.y+y==21))
				change_all=1;

	if (change_all!=1)
		for (x=0;x<4;x++)
			for (y=0;y<4;y++)
				if ((currentPiece.x+x<10)&&(currentPiece.y+y+1<22)&&(currentPiece.piece[x][y]!=0))
					if (tboard[x+currentPiece.x][y+currentPiece.y+1]!=0)
						change_all=1;

	if (change_all==1){
		for (x=0;x<4;x++)
			for (y=0;y<4;y++)
				if ((currentPiece.x+x<10)&&(currentPiece.y+y>=0)&&(currentPiece.piece[x][y]!=0))
					tboard[x+currentPiece.x][y+currentPiece.y]=currentPiece.piece[x][y];

		for (x=0;x<10;x++)
			for (y=0;y<22;y++)
				board[x][y] = tboard[x][y];
		NextPiece();
   }

	for (x=0;x<10;x++)
		for (y=0;y<3;y++)
			if (tboard[x][y]!=0)
				game_over=1;

	y=21;

	if ((change_all==1)&&(game_over!=1))
		while(loop){
			if ((board[0][y]!=0)&&(board[1][y]!=0)&&(board[2][y]!=0)&&(board[3][y]!=0))
				if ((board[4][y]!=0)&&(board[5][y]!=0)&&(board[6][y]!=0)&&(board[7][y]!=0))
					if ((board[8][y]!=0)&&(board[9][y]!=0)){
						for (x=0;x<10;x++)
							for (i=y-1;i>0;i--){
								board[x][i+1]=board[x][i];
								board[x][i]=0;
							}
							y++;
							line++;
					}
			y--;
			if (y==0)
				loop=0;
		}

	if(change_all==0)
		currentPiece.y++;
	//else
		//thinkAI(board, &currentPiece);

	return game_over;
}

void left()
{
	int x,y,can_move=1;
	int tboard [10] [22];

	for (x=0;x<10;x++)
		for (y=0;y<22;y++)
			tboard [x][y] = board [x][y];
	for (x=0;x<4;x++)
		for (y=0;y<4;y++)
			if ((currentPiece.x+x<10)&&(currentPiece.y+y<22)&&(currentPiece.piece[x][y]))
				tboard[x+currentPiece.x][y+currentPiece.y]=1;

	for (y=0;y<22;y++)
		if (tboard[0][y]==1)
			can_move=0;

	for (x=1;x<10;x++)
		for (y=0;y<22;y++)
			if ((tboard[x][y]==1)&&(tboard[x-1][y]>1))
				can_move=0;

	if (can_move==1)
		currentPiece.x--;
}

void right()
{
	int x,y,can_move=1;
	int tboard [10] [22];

	for (x=0;x<10;x++)
		for (y=0;y<22;y++)
			tboard [x][y] = board [x][y];

	for (x=0;x<4;x++)
		for (y=0;y<4;y++)
			if ((currentPiece.x+x<10)&&(currentPiece.y+y<22)&&(currentPiece.piece[x][y]))
				tboard[x+currentPiece.x][y+currentPiece.y]=1;

	for (y=0;y<22;y++)
		if (tboard[9][y]==1)
			can_move=0;

	for (x=0;x<9;x++)
		for (y=0;y<22;y++)
			if ((tboard[x][y]==1)&&(tboard[x+1][y]>1))
				can_move=0;

	if (can_move==1)
		currentPiece.x++;
}

void RotatePiece()
{
	int x,y,dir=1,can_rotate=1;
	char temp [4][4];

	for(x=0;x<4;x++)
		for(y=0;y<4;y++)
			temp[x][y] = 0;

	switch(currentPiece.type) {
		case 1:
			if (dir==1) {
				for(x=0;x<4;x++)
					for(y=0;y<4;y++)
						temp[y][3-x] = currentPiece.piece[x][y];
			}
			else if (dir==2) {
				for(x=0;x<4;x++)
					for(y=0;y<4;y++)
						temp[3-y][x] = currentPiece.piece[x][y];
			}
			break;
		case 2:
			if (dir==1) {
				for(x=1;x<4;x++)
					for(y=0;y<3;y++)
						temp[y+1][3-x] = currentPiece.piece[x][y];
			}
			else if (dir==2) {
				for(x=1;x<4;x++)
					for(y=0;y<3;y++)
						temp[3-y][x-1] = currentPiece.piece[x][y];
			}
			break;
		case 3:
			can_rotate=0;
			break;
	}

	for(x=0;x<4;x++)
		for(y=0;y<4;y++)
			if (temp[x][y]!=0)
				if((x+currentPiece.x<0)||(x+currentPiece.x>9)||(y+currentPiece.y>21)||
					(board[x+currentPiece.x][y+currentPiece.y]!=0))
					can_rotate=0;
	if (can_rotate==1){
		for(x=0;x<4;x++)
			for(y=0;y<4;y++)
				currentPiece.piece[x][y] = temp[x][y];

	}
}